@@ -3,12 +3,15 @@
 
     If you spot any issues, please do not hesitate to send in a Pull Request
     or reach out to me on Discord (LightArrowsEXE#0476)!
+
+    For further support, drop by `#lvsfunc` in the `IEW Discord server <https://discord.gg/qxTxVJGtst>`_.
 """
 
 # flake8: noqa
 
 from . import (aa, comparison, deblock, dehalo, dehardsub, deinterlace,
-               denoise, kernels, mask, misc, recon, render, scale, types, util)
+               denoise, fun, kernels, mask, misc, recon, render, scale, types,
+               util)
 
 # Aliases:
 comp = comparison.compare
