@@ -1,23 +1,25 @@
 """
-    Functions intended to be used to make comparisons between different sources
-    or to be used to analyze something from a single clip.
+    Comparison and analysis functions and wrappers.
+
+    These functions are intended to be used for comparing encodes, filterchains,
+    and other kinds of personal analysis.
 """
 import math
 import random
 import warnings
 from abc import ABC, abstractmethod
 from enum import IntEnum
-from itertools import zip_longest
-from typing import (Any, Callable, Dict, Iterable, Iterator, List,
-                    Optional, Sequence, Set, Tuple, TypeVar, Union)
+from itertools import groupby, zip_longest
+from typing import (Any, Callable, Dict, Iterable, Iterator, List, Literal,
+                    Optional, Sequence, Set, Tuple, TypeVar, Union, overload)
 
 import vapoursynth as vs
 import vsutil
 
 from .dehardsub import hardsub_mask
+from .misc import get_matrix
 from .render import clip_async_render
 from .util import get_prop
-from .misc import get_matrix
 
 core = vs.core
 
@@ -344,6 +346,9 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
     Clips are automatically resampled to 8 bit YUV -> RGB24 to emulate how a monitor shows the frame.
     This can be disabled by setting `force_resample` to ``False``.
 
+    This is not recommended over setting multiple outputs and checking between those,
+    but in the event that is unavailable to you, this function may be useful.
+
     Alias for this function is `lvsfunc.comp`.
 
     :param clip_a:         Clip to compare
@@ -483,46 +488,72 @@ def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> vs.Vid
     return core.std.MaskedMerge(core.std.MakeDiff(a, a), core.std.MakeDiff(a, b), hardsub_mask(a, b))
 
 
+@overload
+def diff(*clips: vs.VideoNode,
+         thr: float = ...,
+         height: int = ...,
+         interleave: bool = ...,
+         return_ranges: Literal[True] = True,
+         exclusion_ranges: Optional[Sequence[Union[int, Tuple[int, int]]]] = ...,
+         diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = ...,
+         **namedclips: vs.VideoNode) -> Tuple[vs.VideoNode, List[Tuple[int, int]]]:
+    ...
+
+
+@overload
+def diff(*clips: vs.VideoNode,
+         thr: float = ...,
+         height: int = ...,
+         interleave: bool = ...,
+         return_ranges: Literal[False],
+         exclusion_ranges: Optional[Sequence[Union[int, Tuple[int, int]]]] = ...,
+         diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = ...,
+         **namedclips: vs.VideoNode) -> vs.VideoNode:
+    ...
+
+
 def diff(*clips: vs.VideoNode,
          thr: float = 72,
          height: int = 288,
-         return_array: bool = False,
-         return_frames: bool = False,
+         interleave: bool = False,
+         return_ranges: bool = False,
+         exclusion_ranges: Optional[Sequence[Union[int, Tuple[int, int]]]] = None,
          diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = lambda a, b: core.std.MakeDiff(a, b),
-         **namedclips: vs.VideoNode) -> Union[vs.VideoNode, Tuple[vs.VideoNode, List[int]]]:
+         **namedclips: vs.VideoNode) -> Union[vs.VideoNode, Tuple[vs.VideoNode, List[Tuple[int, int]]]]:
     """
     Creates a standard :py:class:`lvsfunc.comparison.Stack` between frames from two clips that have differences.
     Useful for making comparisons between TV and BD encodes, as well as clean and hardsubbed sources.
 
-    There are two methods used here to find differences.
+    There are two methods used here to find differences:
     If `thr` is below 1, PlaneStatsDiff is used to figure out the differences.
     Else, if `thr` is equal than or higher than 1, PlaneStatsMin/Max are used.
 
-    Recommended is PlaneStatsMin/Max, as those seem to catch
-    more outrageous differences more easily and not return
-    too many starved frames.
+    Recommended is PlaneStatsMin/Max, as those seem to catch more outrageous differences
+    without returning too many starved frames.
 
     Note that this might catch artifacting as differences!
     Make sure you verify every frame with your own eyes!
 
     Alias for this function is `lvsfunc.diff`.
 
-    :param clips:         Clips for comparison (order is kept)
-    :param namedclips:    Keyword arguments of `name=clip` for all clips in the comparison.
-                          Clips will be labeled at the top left with their `name`.
-    :param thr:           Threshold, <= 1 uses PlaneStatsDiff, >1 uses Max/Min.
-                          Value must be below 128 (Default: 72)
-    :param height:        Height in px to downscale clips to if `return_array` is ``False``
-                          (MakeDiff clip will be twice this resolution) (Default: 288)
-    :param return_array:  Return frames as an interleaved comparison (using :py:class:`lvsfunc.comparison.Interleave`)
-                          (Default: ``False``)
-    :param return_frames: Adds `frames list` to the return. (Default: ``False``)
-    :param diff_func:     Function for calculating diff in PlaneStatsMin/Max mode.
-                          (Default: core.std.MakeDiff)
+    :param clips:               Clips for comparison (order is kept)
+    :param namedclips:          Keyword arguments of `name=clip` for all clips in the comparison.
+                                Clips will be labeled at the top left with their `name`.
+    :param thr:                 Threshold, <= 1 uses PlaneStatsDiff, > 1 uses Max/Min.
+                                Higher values will catch more differences.
+                                Value must be lower than 128
+    :param height:              Height in px to downscale clips to if `interleave` is ``False``
+                                (MakeDiff clip will be twice this resolution)
+    :param interleave:          Return clip as an interleaved comparison
+                                (using :py:class:`lvsfunc.comparison.Interleave`).
+                                This will not return a diff clip
+    :param return_ranges:       Return a list of ranges in addition to the comparison clip
+    :param exclusion_ranges:    Excludes a list of frame ranges from difference checking output (but not processing)
+    :param diff_func:           Function for calculating diff in PlaneStatsMin/Max mode
 
-    :return: Either an interleaved clip of the differences between the two clips
-             or a stack of both input clips on top of MakeDiff clip.
-             Optionally, you can allow the function to also return a list of frames as well.
+    :return:                    Either an interleaved clip of the differences between the two clips
+                                or a stack of both input clips on top of MakeDiff clip.
+                                Furthermore, the function will print the ranges of all the diffs found.
     """
     if clips and namedclips:
         raise ValueError("diff: positional clips and named keyword clips cannot both be given")
@@ -540,6 +571,12 @@ def diff(*clips: vs.VideoNode,
         if any(nc.format is None for nc in namedclips.values()):
             raise ValueError("diff: variable-format namedclips not supported")
 
+    def _to_ranges(iterable: List[int]) -> Iterable[Tuple[int, int]]:
+        iterable = sorted(set(iterable))
+        for key, group in groupby(enumerate(iterable), lambda t: t[1] - t[0]):
+            groupl = list(group)
+            yield groupl[0][1], groupl[-1][1]
+
     if clips:
         a, b = vsutil.depth(clips[0], 8), vsutil.depth(clips[1], 8)
     elif namedclips:
@@ -572,17 +609,24 @@ def diff(*clips: vs.VideoNode,
 
     frames.sort()
 
+    if exclusion_ranges:
+        r: List[int] = []
+        for e in exclusion_ranges:
+            if isinstance(e, int):
+                e = (e, e)
+            start, end = e[0], e[1]+1
+            r += list(range(start, end))
+        frames = [f for f in frames if f not in r]
+
     if clips:
         name_a, name_b = "Clip A", "Clip B"
     else:
         name_a, name_b = namedclips.keys()
 
-    if return_array:
+    if interleave:
         a, b = a.text.FrameNum(9), b.text.FrameNum(9)
         comparison = Interleave({f'{name_a}': core.std.Splice([a[f] for f in frames]),
                                  f'{name_b}': core.std.Splice([b[f] for f in frames])}).clip
-        return comparison if not return_frames else (comparison, frames)
-
     else:
         scaled_width = vsutil.get_w(height, only_even=False)
         diff = diff.resize.Spline36(width=scaled_width * 2, height=height * 2).text.FrameNum(9)
@@ -594,7 +638,11 @@ def diff(*clips: vs.VideoNode,
         diff = core.std.Splice([diff[f] for f in frames])
 
         comparison = Stack((diff_stack, diff), direction=Direction.VERTICAL).clip
-        return comparison if not return_frames else (comparison, frames)
+
+    if return_ranges:
+        return comparison, list(_to_ranges(frames))
+    else:
+        return comparison
 
 
 def interleave(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
