@@ -1,10 +1,10 @@
 """
-Kernels for vapoursynth internal resizers. Intended for use by
-:py:mod:`lvsfunc.scale` functions.
+    Kernels for VapourSynth internal resizers.
+    Intended for use by :py:mod:`lvsfunc.scale` functions.
 """
 from abc import ABC, abstractmethod
 from math import sqrt
-from typing import Any, Tuple
+from typing import Any, Dict, Tuple
 
 import vapoursynth as vs
 
@@ -18,6 +18,10 @@ class Kernel(ABC):
     Additional kwargs supplied to constructor are passed only to the internal
     resizer, not the descale resizer.
     """
+
+    kwargs: Dict[str, Any]
+    """Arguments passed to the kernel filter"""
+
     def __init__(self, **kwargs: Any) -> None:
         self.kwargs = kwargs
 
@@ -216,3 +220,14 @@ class RobidouxSharp(Bicubic):
         b = 6 / (13 + 7 * sqrt(2))
         c = 7 / (2 + 12 * sqrt(2))
         super().__init__(b=b, c=c, **kwargs)
+
+
+class BicubicDidee(Bicubic):
+    """
+    Kernel inspired by a Didée post.
+    See: https://forum.doom9.org/showthread.php?p=1748922#post1748922.
+
+    This is useful for downscaling content, but might not help much with upscaling.
+    """
+    def __init__(self, **kwargs: Any) -> None:
+        super().__init__(b=-1/2, c=1/4, **kwargs)
